import { useEffect, useState } from 'react'

import TagLink from '@/components/TagLink'
import TagSearch from '@/components/TagSearch'
import SubsectionTitle from '@/components/common/SubsectionTitle'
import SidebarComponent from '@/components/layout/SidebarComponent'
import { CATEGORIES } from '@/consts'

import type { Category, Post, Tag } from '@/types'

type TagByCategory = { [key in Category]: Tag[] }

type Props = { post: Post | null }


export default ({ post }: Props) => {
  const [tags, setTags] = useState ({ } as TagByCategory)

  const categoryNames: Record<Category, string> = {
      deerjikist: 'ニジラー',
      meme:       '原作・ネタ元・ミーム等',
      character:  'キャラクター',
      general:    '一般',
      material:   '素材',
      meta:       'メタタグ',
      nico:       'ニコニコタグ' }

  useEffect (() => {
    if (!(post))
      return

    const tagsTmp = { } as TagByCategory

    for (const tag of post.tags)
      {
	if (!(tag.category in tagsTmp))
	  tagsTmp[tag.category] = []
	tagsTmp[tag.category].push (tag)
      }

    for (const cat of Object.keys (tagsTmp) as (keyof typeof tagsTmp)[])
      tagsTmp[cat].sort ((tagA: Tag, tagB: Tag) => tagA.name < tagB.name ? -1 : 1)

    setTags (tagsTmp)
  }, [post])

  return (
      <SidebarComponent>
	<TagSearch />
	{CATEGORIES.map ((cat: Category) => cat in tags && (
	    <div className="my-3" key={cat}>
	      <SubsectionTitle>{categoryNames[cat]}</SubsectionTitle>
	      <ul>
		{tags[cat].map ((tag, i) => (
		    <li key={i} className="mb-1">
		      <TagLink tag={tag} />
		    </li>))}
	      </ul>
	    </div>))}
      </SidebarComponent>)
}
