import { Link } from 'react-router-dom'

import { LIGHT_COLOUR_SHADE, DARK_COLOUR_SHADE, TAG_COLOUR } from '@/consts'
import { cn } from '@/lib/utils'

import type { Tag } from '@/types'

type Props = { tag:         Tag
	       linkFlg?:    boolean
	       withWiki?:   boolean
	       withCount?:  boolean }


export default ({ tag,
		  linkFlg   = true,
		  withWiki  = true,
		  withCount = true }: Props) => {
  const spanClass = cn (
      `text-${ TAG_COLOUR[tag.category] }-${ LIGHT_COLOUR_SHADE }`,
      `dark:text-${ TAG_COLOUR[tag.category] }-${ DARK_COLOUR_SHADE }`)
  const linkClass = cn (
      spanClass,
      `hover:text-${ TAG_COLOUR[tag.category] }-${ LIGHT_COLOUR_SHADE - 200 }`,
      `dark:hover:text-${ TAG_COLOUR[tag.category] }-${ DARK_COLOUR_SHADE - 200 }`)

  return (
      <>
	{(linkFlg && withWiki) && (
	    <span className="mr-1">
	      <Link to={`/wiki/${ encodeURIComponent (tag.name) }`}
		    className={linkClass}>
		?
	      </Link>
	    </span>)}
	{linkFlg
	 ? (
	    <Link to={`/posts?${ (new URLSearchParams ({ tags: tag.name })).toString () }`}
		  className={linkClass}>
	      {tag.name}
	    </Link>)
	 : (
	    <span className={spanClass}>
	      {tag.name}
	    </span>)}
	{withCount && (
	    <span className="ml-1">{tag.postCount}</span>)}
      </>)
}
