import axios from 'axios'
import toCamel from 'camelcase-keys'
import { useState, useEffect } from 'react'
import { Link, useLocation, /* useNavigate */ } from 'react-router-dom'

import { API_BASE_URL } from '@/config'
import { WikiIdBus } from '@/lib/eventBus/WikiIdBus'
import { cn } from '@/lib/utils'

import type { Menu, Tag, User, WikiPage } from '@/types'

type Props = { user: User | null }


export default ({ user }: Props) => {
  const location = useLocation ()
  // const navigate = useNavigate ()

  // const [activeIndex, setActiveIndex] = useState (-1)
  const [postCount, setPostCount] = useState<number | null> (null)
  // const [suggestionsVsbl, setSuggestionsVsbl] = useState (false)
  // const [suggestions, setSuggestions] = useState<WikiPage[]> ([])
  // const [tagSearch, setTagSearch] = useState ('')
  // const [userSearch, setUserSearch] = useState ('')
  const [wikiId, setWikiId] = useState<number | null> (WikiIdBus.get ())
  // const [wikiSearch, setWikiSearch] = useState ('')

  const wikiPageFlg = Boolean (/^\/wiki\/(?!new|changes)[^\/]+/.test (location.pathname) && wikiId)
  const wikiTitle = location.pathname.split ('/')[2]
  const menu: Menu = [
      { name: '広場', to: '/posts', subMenu: [
	  { name: '一覧', to: '/posts' },
	  { name: '投稿追加', to: '/posts/new' },
	  { name: 'ヘルプ', to: '/wiki/ヘルプ:広場' }] },
      { name: 'タグ', to: '/tags', subMenu: [
	  { name: 'タグ一覧', to: '/tags', visible: false },
	  { name: '別名タグ', to: '/tags/aliases', visible: false },
	  { name: '上位タグ', to: '/tags/implications', visible: false },
	  { name: 'ニコニコ連携', to: '/tags/posts' },
	  { name: 'タグのつけ方', to: '/wiki/ヘルプ:タグのつけ方' },
	  { name: 'ヘルプ', to: '/wiki/ヘルプ:タグ' }] },
      { name: 'Wiki', to: '/wiki/ヘルプ:ホーム', base: '/wiki', subMenu: [
	  { name: '検索', to: '/wiki' },
	  { name: '新規', to: '/wiki/new' },
	  { name: '全体履歴', to: '/wiki/changes' },
	  { name: 'ヘルプ', to: '/wiki/ヘルプ:Wiki' },
	  { component: <span className="flex items-center px-2">|</span>,
	    visible: wikiPageFlg },
	  { name: `広場 (${ postCount || 0 })`, to: `/posts?tags=${ wikiTitle }`,
	    visible: wikiPageFlg },
	  { name: '履歴', to: `/wiki/changes?id=${ wikiId }`, visible: wikiPageFlg },
	  { name: '編輯', to: `/wiki/${ wikiId || wikiTitle }/edit`, visible: wikiPageFlg }] },
      { name: 'ユーザ', to: '/users', subMenu: [
	  { name: '一覧', to: '/users', visible: false },
	  { name: 'お前', to: `/users/${ user?.id }`, visible: false },
	  { name: '設定', to: '/users/settings', visible: Boolean (user) }] }]

//  const whenTagSearchChanged = (ev: React.ChangeEvent<HTMLInputElement>) => {
//    // TODO: 実装
//
//    setTagSearch (ev.target.value)
//
//    const q: string = ev.target.value.split (' ').at (-1)
//    if (!(q))
//      {
//	// setSuggestions ([])
//	return
//      }
//  }

//  const whenWikiSearchChanged = (ev: React.ChangeEvent<HTMLInputElement>) => {
//    // TODO: 実装
//
//    setWikiSearch (ev.target.value)
//
//    const q: string = ev.target.value.split (' ').at (-1)
//    if (!(q))
//      {
//	// setSuggestions ([])
//	return
//      }
//  }

//  const whenUserSearchChanged = (ev: React.ChangeEvent<HTMLInputElement>) => {
//    // TODO: 実装
//
//    setUserSearch (ev.target.value)
//
//    const q: string = ev.target.value.split (' ').at (-1)
//    if (!(q))
//      {
//	// setSuggestions ([])
//	return
//      }
//  }

//  const handleKeyDown = (ev: React.KeyboardEvent<HTMLInputElement>) => {
//     if (ev.key === 'Enter' && wikiSearch.length && (!(suggestionsVsbl) || activeIndex < 0))
//       {
//	navigate (`/wiki/${ encodeURIComponent (wikiSearch) }`)
//	setSuggestionsVsbl (false)
//       }
//  }

//  const handleTagSelect = (tag: Tag) => {
//  }

  useEffect (() => {
    const unsubscribe = WikiIdBus.subscribe (setWikiId)
    return () => unsubscribe ()
  }, [])

  useEffect (() => {
    if (!(wikiId))
      return

    const fetchPostCount = async () => {
      try
      {
	const pageRes = await axios.get (`${ API_BASE_URL }/wiki/${ wikiId }`)
	const wikiPage = toCamel (pageRes.data as any, { deep: true }) as WikiPage

	const tagRes = await axios.get (`${ API_BASE_URL }/tags/name/${ wikiPage.title }`)
	const tag = toCamel (tagRes.data as any, { deep: true }) as Tag

	setPostCount (tag.postCount)
      }
      catch
      {
	setPostCount (0)
      }
    }
    fetchPostCount ()
  }, [wikiId])

  return (
      <>
	<nav className="px-3 flex justify-between items-center w-full min-h-[48px]
			bg-yellow-50 dark:bg-red-975">
	  <div className="flex items-center gap-2 h-full">
	    <Link to="/" className="mx-4 text-xl font-bold
				    text-pink-600 hover:text-pink-400
				    dark:text-pink-300 dark:hover:text-pink-100">
	      ぼざクリ タグ広場
	    </Link>
	    {menu.map (item => (
		<Link to={item.to}
		      className={cn ('h-full flex items-center',
				     (location.pathname.startsWith (item.base || item.to)
				      ? 'bg-yellow-200 dark:bg-red-950 px-4 font-bold'
				      : 'px-2'))}>
		  {item.name}
		</Link>
	    ))}
	  </div>
	  {user && (
	      <div className="ml-auto pr-4">
		<Link to="/users/settings"
		      className="font-bold text-red-600 hover:text-red-400
				 dark:text-yellow-400 dark:hover:text-yellow-200">
		  {user.name || '名もなきニジラー'}
		</Link>
	      </div>)}
	</nav>
	<div className={cn ('bg-yellow-200 dark:bg-red-950',
			    'text-white px-3 flex items-center w-full min-h-[40px]')}>
	  {menu.find (item => location.pathname.startsWith (item.base || item.to))?.subMenu
	   .filter (item => item.visible ?? true)
	   .map (item => 'component' in item ? item.component : (
	      <Link to={item.to} className="h-full flex items-center px-3">
		{item.name}
	      </Link>))}
	</div>
      </>)
}
